//Wrapper for PIC32 FatFS implementation
//FatFS available from http://elm-chan.org/fsw/ff/00index_e.html
//Adapted by Aiden Morrison, 2011.


// Ported by Riccardo Leonardi to PIC32MX795F512L  (22/11/2011)
// Many thanks to Aiden Morrison's good work!
// Changes from Aiden Morrison version:
// -> device changed to PIC32MX795F512L (used into chipKIT Max32)
// -> Header files included via "Include Search Path" option in project
// -> System config performance optimization
// -> LEDs removed (I don't have them in my "shield")
// -> FatFs version updated to R0.09 September 6, 2011 (multi-partition)
// -> get_fattime routine moved to main
// -> disk_timerproc call added (handled by ISR) 
// -> parametrization of SPI port number
// -> core_timer added for rtc and benchmarking purposes

#define GetSystemClock() (80000000ul)

#define TOGGLES_PER_SEC			1000
#define CORE_TICK_RATE	       (GetSystemClock()/2/TOGGLES_PER_SEC)

#define  HDP	799
#define  HT		928
#define  HPS	46
#define  LPS	15	
#define  HPW	48

#define  VDP	479
#define  VT		525
#define  VPS	16
#define  FPS	8
#define  VPW 	16



#include <p32xxxx.h>
//#include <osc.h>
//#include <system.h>
#include <plib.h>
#include ".\fatfs\ff.h"

volatile BYTE rtcYear = 111, rtcMon = 11, rtcMday = 22;	// RTC starting values
volatile BYTE rtcHour = 0, rtcMin = 0, rtcSec = 0;
volatile unsigned long tick, tack;	// Used for time benchmarking purposes



//Work registers for fs command
DWORD acc_size;			/* Work register for fs command */
WORD acc_files, acc_dirs;
FILINFO Finfo;
const BYTE ft[] = {0,12,16,32};

//File system object
FATFS Fatfs;
FATFS *fs;				/* Pointer to file system object */
FIL file1, file2;		/* File objects */

//char speedBuffer[]= "IMTOTALLYGOINGSOFAST!!!!!!!!!!!!IMTOTALLYGOINGSOFAST!!!!!!!!!!!!IMTOTALLYGOINGSOFAST!!!!!!!!!!!!IMTOTALLYGOINGSOFAST!!!!!!!!!!!!IMTOTALLYGOINGSOFAST!!!!!!!!!!!!IMTOTALLYGOINGSOFAST!!!!!!!!!!!!IMTOTALLYGOINGSOFAST!!!!!!!!!!!!IMTOTALLYGOINGSOFAST!!!!!!!!!!!!IMTOTALLYGOINGSOFAST!!!!!!!!!!!!IMTOTALLYGOINGSOFAST!!!!!!!!!!!!IMTOTALLYGOINGSOFAST!!!!!!!!!!!!IMTOTALLYGOINGSOFAST!!!!!!!!!!!!IMTOTALLYGOINGSOFAST!!!!!!!!!!!!IMTOTALLYGOINGSOFAST!!!!!!!!!!!!IMTOTALLYGOINGSOFAST!!!!!!!!!!!!IMTOTALLYGOINGSOFAST!!!!!!!!!!!!";
//char bigspeedBuffer[4096];

    #pragma config FPLLMUL  = MUL_20        // PLL Multiplier 20, 2
    #pragma config FPLLIDIV = DIV_2         // PLL Input Divider
    #pragma config FPLLODIV = DIV_1         // PLL Output Divider
    #pragma config FPBDIV   = DIV_2         // Peripheral Clock divisor
    #pragma config FWDTEN   = OFF           // Watchdog Timer
    #pragma config WDTPS    = PS1           // Watchdog Timer Postscale
    #pragma config FCKSM    = CSDCMD        // Clock Switching & Fail Safe Clock Monitor
    #pragma config OSCIOFNC = OFF           // CLKO Enable
    #pragma config POSCMOD  = HS            // Primary Oscillator
    #pragma config IESO     = OFF           // Internal/External Switch-over
    #pragma config FSOSCEN  = OFF           // Secondary Oscillator Enable (KLO was off)
    #pragma config FNOSC    = PRIPLL        // Oscillator Selection
    #pragma config CP       = OFF           // Code Protect
    #pragma config BWP      = OFF           // Boot Flash Write Protect
    #pragma config PWP      = OFF           // Program Flash Write Protect
    #pragma config ICESEL   = ICS_PGx2      // ICE/ICD Comm Channel Select
    #pragma config DEBUG    = OFF          // Background Debugger Enable
    

/*****************************************************************************
 * Function:  		void CoreTimerHandler(void)     
 * PreCondition:    
 * Input:           None
 * Output:          None
 * Side Effects:    
 * Overview:        FatFs requires a 1ms tick timer to aid
 *					with low level function timing
 * Note:            Initial Microchip version adapted to work into ISR routine
 *****************************************************************************/
void __ISR(_CORE_TIMER_VECTOR, IPL2SOFT) CoreTimerHandler(void)
{
	static const BYTE dom[12] = {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};
	static int div1k;
	BYTE n;
	
	// clear the interrupt flag
	mCTClearIntFlag();
    // update the period
    UpdateCoreTimer(CORE_TICK_RATE);

	disk_timerproc();	// call the low level disk IO timer functions
	tick++;				// increment the benchmarking timer
	tack++;	

	// implement a 'fake' RTCC
	if (++div1k >= 1000) {
		div1k = 0;
		if (++rtcSec >= 60) {
			rtcSec = 0;
			if (++rtcMin >= 60) {
				rtcMin = 0;
				if (++rtcHour >= 24) {
					rtcHour = 0;
					n = dom[rtcMon - 1];
					if ((n == 28) && !(rtcYear & 3)) n++;
					if (++rtcMday > n) {
						rtcMday = 1;
						if (++rtcMon > 12) {
							rtcMon = 1;
							rtcYear++;
						}
					}
				}
			}
		}		
	}
}


/*********************************************************************
 * Function:  		DWORD get_fattime(void)
 * PreCondition:    
 * Input:           None
 * Output:          Time
 * Side Effects:    
 * Overview:        when writing fatfs requires a time stamp
 *					in this exmaple we are going to use a counter
 *					If the starter kit has the 32kHz crystal
 *					installed then the RTCC could be used instead
 * Note:           
 ********************************************************************/
DWORD get_fattime(void)
{
	DWORD tmr;

	INTDisableInterrupts();
	tmr =	  (((DWORD)rtcYear - 80) << 25)
			| ((DWORD)rtcMon << 21)
			| ((DWORD)rtcMday << 16)
			| (WORD)(rtcHour << 11)
			| (WORD)(rtcMin << 5)
			| (WORD)(rtcSec >> 1);
	INTEnableInterrupts();

	return tmr;
}


void Init();
void Init_VS();
char write_VS(char x);
void write_SCI_VS(char reg, unsigned short dat);
void write_SDI_VS(char x);
void Init_TFT();
void Address_set(unsigned int x1, unsigned int y1, unsigned int x2, unsigned int y2);
void delayms(unsigned int i);
void LCD_WR_REG(unsigned short reg);
void LCD_WR_Data(short dat);

char x;

void main()
{
Init();
// Enable optimal performance
INTEnableSystemMultiVectoredInt();
SYSTEMConfigPerformance(GetSystemClock());
mOSCSetPBDIV(OSC_PB_DIV_1);				// Use 1:1 CPU Core:Peripheral clocks
OpenCoreTimer(CORE_TICK_RATE);			// Open 1 ms Timer
// set up the core timer interrupt with a prioirty of 2 and zero sub-priority
mConfigIntCoreTimer((CT_INT_ON | CT_INT_PRIOR_2 | CT_INT_SUB_PRIOR_0));


//Initialize Disk
disk_initialize(0);


//Mount Filesystem
f_mount(0, &Fatfs);

FRESULT FOpenRes;
long accessmode;
long accessmode2;
const char backname[] = "TEST.RAW";
const char file1name[]= "TEST.MP3";
accessmode2= (FA_CREATE_ALWAYS | FA_OPEN_ALWAYS | FA_CREATE_NEW | FA_WRITE |FA_READ);
accessmode= (FA_OPEN_ALWAYS | FA_WRITE |FA_READ);

char buffer[32];

UINT read = 0;
delayms(5);

PORTDbits.RD0 = 1; //Hintergrundbel. ein


Init_TFT();


f_open(&file1, backname, (BYTE)accessmode);




int i;

Address_set(0,0,799,479);
for(i=0;i<(800*480);i++)
{
	f_read(&file1, buffer, 2, &read);
	LCD_WR_Data((((short)buffer[1])<<8)+buffer[0]);
}
Address_set(99,99,199,199);
for(i=0;i<(101*101);i++)
{
	
	LCD_WR_Data(0b0000011111100000);
	
	
}

f_close(&file1);

while(1){
	Init_VS();

	UINT i;
	FOpenRes=f_open(&file1, file1name, (BYTE)accessmode); //Open the file to play
	while (1)
	{
	
		FOpenRes=f_read(&file1, buffer, 32, &read);
		if (read == 0)
			break;
		for (i = 0; i<read;i++)
		{
			while(!PORTDbits.RD9);
			write_SDI_VS(buffer[i]);
			
		}
		if (read < 32)
			break;
	}

f_close(&file1);
}

}

void Init()
{
	AD1PCFG=0xffff;
	DDPCONbits.JTAGEN = 0;
	TRISB = 0;
	PORTB = 0;
	TRISD = (1<<9) | (1<<8);
	PORTD = 1;
	TRISG = (1<<7);
	PORTG = 0;
	TRISE = (1<<7);
	PORTE = ~(1<<7);
	PORTGSET = (1<<2) | (1<<3) | (1<<9);
		
}
void Init_VS()
{
	SPI2CON = (1<<15) | (1<<8) | (1<<5);
	//	SPI2BRG = 49;
	SPI2BRG = 800;
	PORTGCLR = (1<<9);
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	PORTGSET = (1<<9);
	write_SCI_VS(0, 0x800);
	write_SCI_VS(3, 0xc000);
	SPI2BRG = 5;
	while (!PORTDbits.RD9);
	/*write_SDI_VS(0x53);
	write_SDI_VS(0xEF);
	write_SDI_VS(0x6E);
	write_SDI_VS(0b01100100);		//This would be the SDI sine test.
	write_SDI_VS(0);
	write_SDI_VS(0);
	write_SDI_VS(0);
	write_SDI_VS(0);*/
}

char write_VS(char x)
{
	SPI2BUF = x;
	while (!SPI2STATbits.SPIRBF);
	return SPI2BUF;
}


void write_SCI_VS(char reg, unsigned short dat)
{
	while (!PORTDbits.RD9)
	PORTGSET = (1<<2);
	PORTGCLR = (1<<3);
	write_VS(2);
	write_VS(reg);
	write_VS((dat>>8)&0xff);
	write_VS(dat&0xFF);
	PORTGSET = (1<<2) | (1<<3);

}

void write_SDI_VS(char x)
{
	PORTGSET = (1<<3);
	PORTGCLR = (1<<2);
	write_VS(x);
	PORTGSET = (1<<2) | (1<<3);
}


void Init_TFT()
{
	PORTE |= 0b1111111;
	delayms(5);
	PORTE &= ~(1<<1);
	PORTE |= 0b1111111;
	delayms(5);
	PORTECLR = (1<<4);
	delayms(1);
	LCD_WR_REG(0x00E2);	//PLL multiplier, set PLL clock to 120M
	LCD_WR_Data(0x0023);	    //N=0x36 for 6.5M, 0x23 for 10M crystal
	LCD_WR_Data(0x0002);
	LCD_WR_Data(0x0004);
	LCD_WR_REG(0x00E0);  // PLL enable
	LCD_WR_Data(0x0001);
	delayms(1);
	LCD_WR_REG(0x00E0);
	LCD_WR_Data(0x0003);
	delayms(5);
	LCD_WR_REG(0x0001);  // software reset
	delayms(5);
	LCD_WR_REG(0x00E6);	//PLL setting for PCLK, depends on resolution
	LCD_WR_Data(0x0003);
	LCD_WR_Data(0x00ff);
	LCD_WR_Data(0x00ff);

	LCD_WR_REG(0x00B0);	//LCD SPECIFICATION
	LCD_WR_Data(0x0027);
	LCD_WR_Data(0x0000);
	LCD_WR_Data((HDP>>8)&0X00FF);  //Set HDP
	LCD_WR_Data(HDP&0X00FF);
    LCD_WR_Data((VDP>>8)&0X00FF);  //Set VDP
	LCD_WR_Data(VDP&0X00FF);
    LCD_WR_Data(0x0000);

	LCD_WR_REG(0x00B4);	//HSYNC
	LCD_WR_Data((HT>>8)&0X00FF);  //Set HT
	LCD_WR_Data(HT&0X00FF);
	LCD_WR_Data((HPS>>8)&0X00FF);  //Set HPS
	LCD_WR_Data(HPS&0X00FF);
	LCD_WR_Data(HPW);			   //Set HPW
	LCD_WR_Data((LPS>>8)&0X00FF);  //Set HPS
	LCD_WR_Data(LPS&0X00FF);
	LCD_WR_Data(0x0000);

	LCD_WR_REG(0x00B6);	//VSYNC
	LCD_WR_Data((VT>>8)&0X00FF);   //Set VT
	LCD_WR_Data(VT&0X00FF);
	LCD_WR_Data((VPS>>8)&0X00FF);  //Set VPS
	LCD_WR_Data(VPS&0X00FF);
	LCD_WR_Data(VPW);			   //Set VPW
	LCD_WR_Data((FPS>>8)&0X00FF);  //Set FPS
	LCD_WR_Data(FPS&0X00FF);

	LCD_WR_REG(0x00BA);
	LCD_WR_Data(0x000F);    //GPIO[3:0] out 1

	LCD_WR_REG(0x00B8);
	LCD_WR_Data(0x0007);    //GPIO3=input, GPIO[2:0]=output
	LCD_WR_Data(0x0001);    //GPIO0 normal

	LCD_WR_REG(0x0036); //rotation
	LCD_WR_Data(0x0000);

	LCD_WR_REG(0x00F0); //pixel data interface
	LCD_WR_Data(0x0003);


	delayms(5);

	LCD_WR_REG(0x0029); //display on



	LCD_WR_REG(0x00d0); 
	LCD_WR_Data(0x000d);
}

void delayms(unsigned int i)
{
	tack = 0;
	while(tack < i);
}
void Address_set(unsigned int x1,unsigned int y1,unsigned int x2,unsigned int y2)
{

	LCD_WR_REG(0x002A);	
	LCD_WR_Data(x1>>8);	    
	LCD_WR_Data(x1&0x00ff);
	LCD_WR_Data(x2>>8);	    
	LCD_WR_Data(x2&0x00ff);
    LCD_WR_REG(0x002b);	
	LCD_WR_Data(y1>>8);	    
	LCD_WR_Data(y1&0x00ff);
	LCD_WR_Data(y2>>8);	    
	LCD_WR_Data(y2&0x00ff);
	LCD_WR_REG(0x002c);	
}
void LCD_WR_REG(unsigned short reg)
{
	PORTECLR = (1<<5);
	PORTB = reg;
	PORTECLR = (1<<3);
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	delayms(1);
	asm("nop");
	PORTESET = (1<<3);

}
void LCD_WR_Data(short dat)
{
	PORTESET = (1<<5) | (1<<1);
	PORTB = dat;
	PORTECLR = (1<<3);


	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");	
asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	PORTESET = (1<<3);
	
}